""" Schemas for form input and validation """

import deform
import colander
from pyramid.csrf import get_csrf_token, check_csrf_token


@colander.deferred
def deferred_csrf_default(node, kw):
    """ sets the current csrf token """
    request = kw.get("request")
    return get_csrf_token(request)


@colander.deferred
def deferred_csrf_validator(node, kw):
    """ validates a submitted csrf token """

    def validate_csrf(node, value):
        request = kw.get("request")
        if not check_csrf_token(request, raises=False):
            raise colander.Invalid(node, "Bad CSRF token")

    return validate_csrf


# Base Schema


class CSRFSchema(colander.Schema):
    """ base class for schemas with csrf validation """

    csrf_token = colander.SchemaNode(
        colander.String(),
        default=deferred_csrf_default,
        validator=deferred_csrf_validator,
        widget=deform.widget.HiddenWidget(),
    )

    @classmethod
    def as_form(cls, request, **kwargs):
        """ returns the schema as a form """
        url = kwargs.pop("url", None)
        if not url:
            url = request.resource_url(request.context, request.view_name)
        schema = cls().bind(request=request)
        settings = {"template": "form.pt"}
        settings.update(kwargs)
        return deform.Form(schema, action=url, **settings)
